/***********************************************************//**
 *
 * Implementation of G1226
 * @author Shinichi Fujino
 * @since 2022.4.1
 * @version 1.0.1.0
 */

/*

CS    _|~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|____

SCLK  ____|~|_|~|_|~|_|~|_|~|_|~|_|~|_|~|_|~|_|~|_|~|_|~|_|~|_|~|_|~|_|~|_|~|_|~|_|~|_|~|_|~|_|~|_|~|_|~|_

SID     |sync(11111b)       |RW |RS | 0 |D7 |D6 |D5 |D4 | 0 | 0 | 0 | 0 |D3 |D2 |D1 |D0 | 0 | 0 | 0 | 0 |


CPHA = 0
CPOL = 0

*/

///////////////////////////////////////////////////////
// include
#include <stdio.h>
#include "g1226.h"

///////////////////////////////////////////////////////
// define
#define G1226_SYNC_BIT		(0xF8)

#define G1226_INST_REG		(0x00)
#define G1226_DATA_REG		(0x02)

#define G1226_INST_DISPCLEAR		(0x01)

#define G1226_INST_HOME		(0x02)

#define G1226_INST_ENTRY		(0x04)
#define G1226_INST_ENTRY_ACINC		(0x02)
#define G1226_INST_ENTRY_ACDEC		(0x00)

#define G1226_INST_CTRL		(0x08)
#define G1226_INST_CTRL_DISP_ON		(0x04)
#define G1226_INST_CTRL_DISP_OFF		(0x00)
#define G1226_INST_CTRL_CUR_ON		(0x02)
#define G1226_INST_CTRL_CUR_OFF		(0x00)
#define G1226_INST_CTRL_BLINK_ON		(0x01)
#define G1226_INST_CTRL_BLINK_OFF		(0x00)

#define G1226_INST_DDRAM		(0x80)
#define G1226_INST_DDRAM_LINE0		(0x80)
#define G1226_INST_DDRAM_LINE1		(0x90)
#define G1226_INST_DDRAM_LINE2		(0x88)
#define G1226_INST_DDRAM_LINE3		(0x98)

#define G1226_STATE_MASK_MODE		(0x0001)
#define G1226_STATE_SHIFT_MODE		(0)

#define G1226_STATE_MASK_ACT		(0x0002)
#define G1226_STATE_SHIFT_ACT		(1)

#define G1226_STATE_MASK_RELAY		(0x0004)
#define G1226_STATE_SHIFT_RELAY		(2)



/***********************************************************//**
 *
 * @brief CN10-3/PB8@nucleo = CS@G1226 = L
 */

#define G1226_disconnect()		HAL_GPIO_WritePin( GPIOB, GPIO_PIN_8, GPIO_PIN_RESET )


/***********************************************************//**
 *
 * @brief CN10-3/PB8@nucleo = CS@G1226 = H
 */

#define G1226_connect()		HAL_GPIO_WritePin( GPIOB, GPIO_PIN_8, GPIO_PIN_SET )


/***********************************************************//**
 *
 * @var G1226_thres_lv
 * @brief Threshold Level
 */

const int16_t G1226_thres_lv[8] =
{
	1433, // >>>>>>>>
	1024, // >>>>>>>
	716, // >>>>>>
	512, // >>>>>
	358, // >>>>
	256, // >>>
	179, // >>
	128, // >
};


/***********************************************************//**
 *
 * @var G1226_str_meter
 * @brief Level Meter
 */

const char* G1226_str_meter[9] =
{
	">>>>>>>>",
	">>>>>>> ",
	">>>>>>  ",
	">>>>>   ",
	">>>>    ",
	">>>     ",
	">>      ",
	">       ",
	"        ",
};


/***********************************************************//**
 *
 * @brief wait 100[usec]
 */

static void G1226_wait100us( int count )
{
	int i;
	
	// possible to check the number of cycles
	CoreDebug->DEMCR |= CoreDebug_DEMCR_TRCENA_Msk;
	DWT->CTRL |= DWT_CTRL_CYCCNTENA_Msk;
	
	for( i = 0; i < count; i++ )
	{
		int32_t start = DWT->CYCCNT; // start cycle
		
		while( 1 )
		{
			int32_t stop = DWT->CYCCNT; // current cycle
			int32_t cycle = stop - start;
			
			if( cycle > 8400 ) // 100[usec]@84[MHz]
			{
				break;
			}
		}
	}
	
	return;
}


/***********************************************************//**
 *
 * @brief Write Instructions Register(IR)
 */

static void G1226_writeInst( G1226_Obj* thisp, uint8_t data )
{
	uint8_t txdata[3];
	
	txdata[0] = G1226_SYNC_BIT | G1226_INST_REG;
	txdata[1] = data & 0xF0;
	txdata[2] = data << 4;
	
	HAL_SPI_Transmit( thisp->hspi, txdata, 3, 0xFFFFFFFF );
	
	return;
}


/***********************************************************//**
 *
 * @brief Write Data Register(DR)
 */

static void G1226_writeData( G1226_Obj* thisp, uint8_t data )
{
	uint8_t txdata[3];
	
	txdata[0] = G1226_SYNC_BIT | G1226_DATA_REG;
	txdata[1] = data & 0xF0;
	txdata[2] = data << 4;
	
	HAL_SPI_Transmit( thisp->hspi, txdata, 3, 0xFFFFFFFF );
	
	return;
}


/***********************************************************//**
 *
 * @brief Initialization
 */

void G1226_reset( G1226_Obj* thisp )
{
	G1226_disconnect();
	
	G1226_wait100us( 1000 ); // 100[ms]
	
	// Display Control
	//   D=1 : Display ON
	//   C=0 : Cursol OFF
	//   B=0 : Blink OFF
	G1226_connect();
	
	G1226_writeInst( thisp, G1226_INST_CTRL | G1226_INST_CTRL_DISP_ON | G1226_INST_CTRL_CUR_OFF | G1226_INST_CTRL_BLINK_OFF );
	
	G1226_disconnect();
	
	G1226_wait100us( 10 ); // 1[ms]
	
	
	// Display Clear
	G1226_connect();
	
	G1226_writeInst( thisp, G1226_INST_DISPCLEAR );
	
	G1226_disconnect();
	
	G1226_wait100us( 1000 ); // 100[ms]
	
	
	// Entry Mode Set
	//   I/D=1 : AC(Address Counter) is increment
	G1226_connect();
	
	G1226_writeInst( thisp, G1226_INST_ENTRY | G1226_INST_ENTRY_ACINC );
	
	G1226_disconnect();
	
	G1226_wait100us( 10 ); // 1[ms]
	
	
	// Return Home
	G1226_connect();
	
	G1226_writeInst( thisp, G1226_INST_HOME );
	
	G1226_disconnect();
	
	G1226_wait100us( 10 ); // 1[ms]
	
	return;
}


/***********************************************************//**
 *
 * @brief Open G1226
 */

void G1226_open( G1226_Obj* thisp, G1226_Params* params )
{
	thisp->hspi = params->hspi;
	
	__HAL_SPI_ENABLE( thisp->hspi );
	
	G1226_reset( thisp );
	
	return;
}


/***********************************************************//**
 *
 * @brief Display
 * @param line Line to display 0~3
 * @param string String to display
 * @param count Count of string
 */

void G1226_print( G1226_Obj* thisp, int line, const int8_t* string, int count )
{
	// Return Home
	G1226_connect();
	
	G1226_writeInst( thisp, G1226_INST_HOME );
	
	G1226_disconnect();
	
	G1226_wait100us( 1 ); // 0.1[ms]
	
	// Set Address Counter
	G1226_connect();
	
	if( line == 0 )
	{
		G1226_writeInst( thisp, G1226_INST_DDRAM_LINE0 );
	}
	else if( line == 1 )
	{
		G1226_writeInst( thisp, G1226_INST_DDRAM_LINE1 );
	}
	else if( line == 2 )
	{
		G1226_writeInst( thisp, G1226_INST_DDRAM_LINE2 );
	}
	else if( line == 3 )
	{
		G1226_writeInst( thisp, G1226_INST_DDRAM_LINE3 );
	}
	
	// Set String
	while( 1 )
	{
		if( count <= 0 ) break;
		
		G1226_writeData( thisp, *string++ );
		
		count -= 1;
	}
	
	G1226_disconnect();
	
	G1226_wait100us( 1 ); // 0.1[ms]
	
	return;
}


/***********************************************************//**
 *
 * @brief Display Clock
 * @param bcdHours RTC Time Hour
 * @param bcdMinutes RTC Time Minutes
 * @param bcdSeconds RTC Time Seconds
 */

void G1226_dispClock( G1226_Obj* thisp, uint8_t bcdHours, uint8_t bcdMinutes, uint8_t bcdSeconds )
{
	int8_t str_time[8];
	
	str_time[0] = '0' + ( bcdHours >> 4 );
	str_time[1] = '0' + ( bcdHours & 0x0F );
	str_time[2] = ':';
	str_time[3] = '0' + ( bcdMinutes >> 4 );
	str_time[4] = '0' + ( bcdMinutes & 0x0F );
	str_time[5] = ':';
	str_time[6] = '0' + ( bcdSeconds >> 4 );
	str_time[7] = '0' + ( bcdSeconds & 0x0F );
	
	G1226_print( thisp, 0, str_time, 8 );
	
	return;
}


/***********************************************************//**
 *
 * @brief Display Level Meter
 * @param level Input level 0~2048
 */

void G1226_dispLevel( G1226_Obj* thisp, int16_t level )
{
	int idx;
	
	for( idx = 0; idx < 8; idx++ )
	{
		if( level >= G1226_thres_lv[idx] )
		{
			break;
		}
	}
	
	G1226_print( thisp, 2, (const int8_t*)G1226_str_meter[idx], 8 );
	
	return;
}




/*
 * Local Variables:
 *  tab-width : 4
 * End:
 *
 * EOF */
