/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * Copyright (c) 2022 STMicroelectronics.
  * All rights reserved.
  *
  * This software is licensed under terms that can be found in the LICENSE file
  * in the root directory of this software component.
  * If no LICENSE file comes with this software, it is provided AS-IS.
  *
  ******************************************************************************
  */

/*

                 Nucleo-64
               --------------------
              |B1-->PC13           |
              |                    |
        CN7-38|                    |
MIC --------->|PC0/ADC1_IN10       |
              |                    |CN10-4
              |        PC6/TIM3_CH1|--------->           Pi3 Model B
              |                    |                    -----------
              |                    |CN10-5        J8-12|
              |                 PB9|------------------>|GPIO18
              |                    |                    -----------
              |                    |
              |                    |                     G1226
              |                    |                    -----------
              |                    |CN10-3           15|
              |                 PB8|------------------>|RS/CS
              |                    |                   |
              |                    |CN10-11          16|
              |        PA5/SPI1_SCK|------------------>|E/SCLK
              |                    |                   |
              |                    |CN10-15          14|
              |       PA7/SPI1_MOSI|------------------>|RW/SID
              |                    |                    -----------
              |                    |



CPU:84[MHz]
ADC:FS=12kHz, right justified, unsigned, 12[bit]
UART2:115200, 8N1

*/

/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include <stdio.h>
#include "stdbool.h"
#include "g1226.h"
#include "peak.h"
#include "util.h"
#include "jihoudet_def.h"
#include "jihoudet_api.h"

/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

#define L_FRAME	(128) /**< Frame length of "jihoudet" */

/***********************************************************//**
 *
 * @struct CMainApp_Obj
 * @brief 
 *
 */

typedef struct
{
	uint8_t adc_notify; /**< Notification of ADC receive full */
	uint8_t adc_bank; /**< Bank of ADC receive */
	uint8_t adc_idx; /**< Index of ADC receive */
	uint16_t adc_u12[2][L_FRAME]; /**< Buffer of ADC receive. Right justified, unsigned, 12[bit] */
	
	char str_clock[8];
	uint8_t uart2_notify; /**< Notification of UART2 receive full */
	
	uint8_t gpio13_notify; /**< Notification of GPIO13 receive full */
	
	G1226_Obj g1226; /**< G1226 */
	Peak_Obj peak; /**< Peak(ADC) */
	CJihouDet_Obj jihoudet; /**< JihouDet */
} MainApp_Obj;


/***********************************************************//**
 *
 * @struct MODULE_BUILD
 * @brief Build information
 *
 */
typedef struct
{
	int8_t date[16]; /**< Build date */
	int8_t time[16]; /**< Build time */
	int8_t author[32]; /**< Author */
	int8_t copyright[128]; /**< Copyright */
	int8_t brief[128]; /**< Module overview */
} MODULE_BUILD;


/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
#define __HAL_ADC_GET_DR(__HANDLE__) ((__HANDLE__)->Instance->DR)

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
ADC_HandleTypeDef hadc1;

RTC_HandleTypeDef hrtc;

SPI_HandleTypeDef hspi1;

TIM_HandleTypeDef htim3;

UART_HandleTypeDef huart2;

/* USER CODE BEGIN PV */

/***********************************************************//**
 *
 * @var mainapp
 * @brief 
 *
 */

MainApp_Obj mainapp;



/***********************************************************//**
 *
 * @var module_version
 * @brief Build information
 *
 */

const MODULE_BUILD module_build =
{
	__DATE__,
	__TIME__,
	"Shinichi Fujino",
	"Copyright (c) 2022. All rights reserved.",
	"Demonstration device for time signal detection",
};


/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_USART2_UART_Init(void);
static void MX_ADC1_Init(void);
static void MX_TIM3_Init(void);
static void MX_SPI1_Init(void);
static void MX_RTC_Init(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */


/***********************************************************//**
 *
 * @brief 
 */


#ifdef __GNUC__
int __io_putchar(int ch)
#else
int fputc(int ch, FILE *f)
#endif
{
    HAL_UART_Transmit(&huart2, (uint8_t *)&ch, 1, 0xFFFF);
    
    return ch;
}


/***********************************************************//**
 *
 * @brief Rx Transfer completed callbacks
 */

void HAL_UART_RxCpltCallback( UART_HandleTypeDef* huart )
{
	mainapp.uart2_notify = 1; // Notification
	
	return;
}

/***********************************************************//**
 *
 * @brief Regular conversion complete callback in non blocking mode
 */

void HAL_ADC_ConvCpltCallback( ADC_HandleTypeDef* hadc )
{
	uint32_t flag, it;
	
	flag = __HAL_ADC_GET_FLAG( hadc, ADC_FLAG_EOC );
	
	it = __HAL_ADC_GET_IT_SOURCE( hadc, ADC_IT_EOC );
	
	if( flag && it )
	{
		uint32_t bank = mainapp.adc_bank;
		uint32_t idx = mainapp.adc_idx;
		
		mainapp.adc_u12[bank][idx] = __HAL_ADC_GET_DR( hadc );
	    idx += 1;
		
		if( idx == L_FRAME )
		{
			bank ^= 1; // Bank switching
			idx = 0; // Index
			mainapp.adc_notify = 0; // Notification
		}
		
		mainapp.adc_bank = (uint8_t)bank;
		mainapp.adc_idx = (uint8_t)idx;
	}
	
	return;
}


/***********************************************************//**
 *
 * @brief EXTI line detection callbacks
 */

void HAL_GPIO_EXTI_Callback(uint16_t gpio_pin)
{
	if( gpio_pin == GPIO_PIN_13 )
	{
		mainapp.gpio13_notify = 1; // Notification
	}
	
	return;
}

/***********************************************************//**
 *
 * @brief wait 1[msec]
 */

static void wait1ms( int count )
{
	int i;
	
	// possible to check the number of cycles
	CoreDebug->DEMCR |= CoreDebug_DEMCR_TRCENA_Msk;
	DWT->CTRL |= DWT_CTRL_CYCCNTENA_Msk;
	
	for( i = 0; i < count; i++ )
	{
		int32_t start = DWT->CYCCNT; // start cycle
		
		while( 1 )
		{
			int32_t stop = DWT->CYCCNT; // current cycle
			int32_t cycle = stop - start;
			
			if( cycle > 84000 ) // 1000[usec]@84[MHz]
			{
				break;
			}
		}
	}
	
	return;
}


/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */
  uint16_t disp_count;
  int16_t led_count;
  uint8_t pre_second;

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_USART2_UART_Init();
  MX_ADC1_Init();
  MX_TIM3_Init();
  MX_SPI1_Init();
  MX_RTC_Init();
  /* USER CODE BEGIN 2 */
	
	wait1ms( 100 ); // 100[ms]
	
	printf( "@date %s, %s\n", module_build.date, module_build.time );
	printf( "@author %s\n", module_build.author );
	printf( "@copyright %s\n", module_build.copyright );
	printf( "@brief %s\n", module_build.brief );
	
	printf( "\nB1 Button : Cancel" );
	
	// Setup RTC
	while( 1 )
	{
		int ok = 1;
		uint8_t decHours;
		uint8_t decMinutes;
		
		mainapp.gpio13_notify = 0;
		mainapp.uart2_notify = 0;
		
		printf( "\nSETUP HH:MM?\n" );
		
		HAL_UART_Receive_IT( &huart2, (uint8_t*)mainapp.str_clock, 6 );
		
		while( 1 )
		{
			if( mainapp.uart2_notify )
			{
				HAL_UART_AbortReceive( &huart2 );
				
				break;
			}
			
			if( mainapp.gpio13_notify )
			{
				// Set to 00:00 when the B1 button is pressed
				mainapp.str_clock[0] = '0';
				mainapp.str_clock[1] = '0';
				mainapp.str_clock[2] = ':';
				mainapp.str_clock[3] = '0';
				mainapp.str_clock[4] = '0';
				mainapp.str_clock[5] = '\n';
				break;
			}
		}
		
		// string -> decimal
		decHours = ( ( mainapp.str_clock[0] - '0' ) * 10 ) + ( mainapp.str_clock[1] - '0' );
		decMinutes = ( ( mainapp.str_clock[3] - '0' ) * 10 ) + ( mainapp.str_clock[4] - '0' );
		
		// error check
		if( decHours >= 24 ) ok = 0;
		if( decMinutes >= 60 ) ok = 0;
		if( mainapp.str_clock[2] != ':' ) ok = 0;
		if( mainapp.str_clock[5] != '\n' ) ok = 0;
		
		if( ok )
		{
			break;
		}
	}
	
	{
		RTC_TimeTypeDef rtc_time;
		RTC_DateTypeDef rtc_date;
		
		HAL_RTC_GetDate( &hrtc, &rtc_date, RTC_FORMAT_BCD );
		
		rtc_time.Hours = ( ( mainapp.str_clock[0] - '0' ) * 16 ) + ( mainapp.str_clock[1] - '0' );
		rtc_time.Minutes = ( ( mainapp.str_clock[3] - '0' ) * 16 ) + ( mainapp.str_clock[4] - '0' );
		rtc_time.Seconds = 0x01;
		rtc_time.DayLightSaving = RTC_DAYLIGHTSAVING_NONE;
		rtc_time.StoreOperation = RTC_STOREOPERATION_RESET;
		
		HAL_RTC_SetTime( &hrtc, &rtc_time, RTC_FORMAT_BCD );
		HAL_RTC_SetDate( &hrtc, &rtc_date, RTC_FORMAT_BCD );
	}
	
	// Starts the TIM Input Capture measurement in interrupt mode
	HAL_TIM_IC_Start_IT( &htim3, TIM_CHANNEL_1 );
	
	// Enables the interrupt and starts ADC
	{
		HAL_ADC_Start_IT( &hadc1 );
		
		mainapp.adc_bank = 0;
		mainapp.adc_idx = 0;
		mainapp.adc_notify = 1;
	}
	
	// Open G1226
	{
		G1226_Params params;
		
		params.hspi = &hspi1;
		
		G1226_open( &mainapp.g1226, &params );
	}
	
	// Reset Peak
	{
		Peak_Params params;
		
		Peak_reset( &mainapp.peak, &params );
	}
	
	// Reset JihouDet
	{
		CJihouDet_Params params;
		
		params.dccut_prm.alpha = (float)0.125; /**< Time constant. (effect:small)0 < a < 1.0(effect:great) */
		params.judge_prm.thres_det = (float)2.0; /**< Sound threshold. 0[dB] : 825.0, -20[dB] : 8.25, -26[dB] : 2.0 */
		
		CJihouDet_reset( &mainapp.jihoudet, &params );
	}
	
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  
	disp_count = 12 - 1; // 128[ms] period
	led_count = -1;
	pre_second = 255;
	
	// PB9/CN10-5 = H
	HAL_GPIO_WritePin( GPIOB, GPIO_PIN_9, GPIO_PIN_SET );
	
	while (1)
	{
		if( mainapp.adc_notify == 0 )
		{
			// 10.6666[ms] period
			bool detect;
			float adc_float[L_FRAME];
			
			mainapp.adc_notify = 1;
			
			// PC5/CN10-6 = L
			HAL_GPIO_WritePin( GPIOC, GPIO_PIN_5, GPIO_PIN_RESET );
			
			// 
			Peak_set( &mainapp.peak, mainapp.adc_u12[mainapp.adc_bank ^ 1], L_FRAME );
			
			// unsigned 12[bit] -> float (-1.0~1.0)
			norm_u12_float( mainapp.adc_u12[mainapp.adc_bank ^ 1], adc_float, L_FRAME );
			
			// 82000[cycle], 7.68[MIPS]
			detect = CJihouDet_apply( &mainapp.jihoudet, adc_float );
			
			// 
			disp_count -= 1;
			
			if( detect )
			{
				// Adjust Clock
				RTC_TimeTypeDef rtc_time;
				RTC_DateTypeDef rtc_date;
				
				HAL_RTC_GetTime( &hrtc, &rtc_time, RTC_FORMAT_BIN );
				HAL_RTC_GetDate( &hrtc, &rtc_date, RTC_FORMAT_BCD );
				
				if( rtc_time.Minutes >= 30 )
				{
					rtc_time.Hours += 1;
					if( rtc_time.Hours >= 24 ) rtc_time.Hours = 0;
				}
				
				rtc_time.Minutes = 0;
				rtc_time.Seconds = 1;
				rtc_time.DayLightSaving = RTC_DAYLIGHTSAVING_NONE;
				rtc_time.StoreOperation = RTC_STOREOPERATION_RESET;
				
				HAL_RTC_SetTime( &hrtc, &rtc_time, RTC_FORMAT_BIN );
				HAL_RTC_SetDate( &hrtc, &rtc_date, RTC_FORMAT_BCD );
				
				// PB9/CN10-5 = L
				HAL_GPIO_WritePin( GPIOB, GPIO_PIN_9, GPIO_PIN_RESET );
				
				disp_count = 12 - 1;
				
				led_count = 375 - 1; // 4000[ms]
			}
			else if( disp_count == 0 )
			{
				// LCD Clock update
				RTC_TimeTypeDef rtc_time;
				RTC_DateTypeDef rtc_date;
				int16_t adc_max;
				
				disp_count = 12 - 1;
				
				// LCD Level meter update
				adc_max = Peak_get( &mainapp.peak );
				
				G1226_dispLevel( &mainapp.g1226, adc_max );
				
				HAL_RTC_GetTime( &hrtc, &rtc_time, RTC_FORMAT_BCD );
				HAL_RTC_GetDate( &hrtc, &rtc_date, RTC_FORMAT_BCD );
				
				// when Seconds changes
				if( pre_second != rtc_time.Seconds )
				{
					char str_time[16];
					
					pre_second = rtc_time.Seconds;
					
					// LCD Clock update when Seconds changes
					G1226_dispClock( &mainapp.g1226, rtc_time.Hours, rtc_time.Minutes, rtc_time.Seconds );
					
					// Send to UART2 when Seconds changes
					str_time[0] = '0' + ( rtc_time.Hours >> 4 );
					str_time[1] = '0' + ( rtc_time.Hours & 0x0F );
					str_time[2] = ':';
					str_time[3] = '0' + ( rtc_time.Minutes >> 4 );
					str_time[4] = '0' + ( rtc_time.Minutes & 0x0F );
					str_time[5] = ':';
					str_time[6] = '0' + ( rtc_time.Seconds >> 4 );
					str_time[7] = '0' + ( rtc_time.Seconds & 0x0F );
					str_time[8] = '\n';
					str_time[9] = '\0';
					
					printf( str_time );
				}
			}
			
			if( led_count >= 0 )
			{
				if( led_count == 0 )
				{
					// PB9/CN10-5 = H
					HAL_GPIO_WritePin( GPIOB, GPIO_PIN_9, GPIO_PIN_SET );
				}
				
				led_count -= 1;
			}
			
			// PC5 = H
 			HAL_GPIO_WritePin( GPIOC, GPIO_PIN_5, GPIO_PIN_SET );
		}
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
	}
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Configure the main internal regulator output voltage
  */
  __HAL_RCC_PWR_CLK_ENABLE();
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE3);

  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI|RCC_OSCILLATORTYPE_LSE;
  RCC_OscInitStruct.LSEState = RCC_LSE_ON;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
  RCC_OscInitStruct.PLL.PLLM = 16;
  RCC_OscInitStruct.PLL.PLLN = 336;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV4;
  RCC_OscInitStruct.PLL.PLLQ = 2;
  RCC_OscInitStruct.PLL.PLLR = 2;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief ADC1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_ADC1_Init(void)
{

  /* USER CODE BEGIN ADC1_Init 0 */

  /* USER CODE END ADC1_Init 0 */

  ADC_ChannelConfTypeDef sConfig = {0};

  /* USER CODE BEGIN ADC1_Init 1 */

  /* USER CODE END ADC1_Init 1 */

  /** Configure the global features of the ADC (Clock, Resolution, Data Alignment and number of conversion)
  */
  hadc1.Instance = ADC1;
  hadc1.Init.ClockPrescaler = ADC_CLOCK_SYNC_PCLK_DIV4;
  hadc1.Init.Resolution = ADC_RESOLUTION_12B;
  hadc1.Init.ScanConvMode = DISABLE;
  hadc1.Init.ContinuousConvMode = DISABLE;
  hadc1.Init.DiscontinuousConvMode = DISABLE;
  hadc1.Init.ExternalTrigConvEdge = ADC_EXTERNALTRIGCONVEDGE_RISINGFALLING;
  hadc1.Init.ExternalTrigConv = ADC_EXTERNALTRIGCONV_T3_CC1;
  hadc1.Init.DataAlign = ADC_DATAALIGN_RIGHT;
  hadc1.Init.NbrOfConversion = 1;
  hadc1.Init.DMAContinuousRequests = DISABLE;
  hadc1.Init.EOCSelection = ADC_EOC_SINGLE_CONV;
  if (HAL_ADC_Init(&hadc1) != HAL_OK)
  {
    Error_Handler();
  }

  /** Configure for the selected ADC regular channel its corresponding rank in the sequencer and its sample time.
  */
  sConfig.Channel = ADC_CHANNEL_10;
  sConfig.Rank = 1;
  sConfig.SamplingTime = ADC_SAMPLETIME_3CYCLES;
  if (HAL_ADC_ConfigChannel(&hadc1, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN ADC1_Init 2 */

  /* USER CODE END ADC1_Init 2 */

}

/**
  * @brief RTC Initialization Function
  * @param None
  * @retval None
  */
static void MX_RTC_Init(void)
{

  /* USER CODE BEGIN RTC_Init 0 */

  /* USER CODE END RTC_Init 0 */

  RTC_TimeTypeDef sTime = {0};
  RTC_DateTypeDef sDate = {0};

  /* USER CODE BEGIN RTC_Init 1 */

  /* USER CODE END RTC_Init 1 */

  /** Initialize RTC Only
  */
  hrtc.Instance = RTC;
  hrtc.Init.HourFormat = RTC_HOURFORMAT_24;
  hrtc.Init.AsynchPrediv = 127;
  hrtc.Init.SynchPrediv = 255;
  hrtc.Init.OutPut = RTC_OUTPUT_DISABLE;
  hrtc.Init.OutPutPolarity = RTC_OUTPUT_POLARITY_HIGH;
  hrtc.Init.OutPutType = RTC_OUTPUT_TYPE_OPENDRAIN;
  if (HAL_RTC_Init(&hrtc) != HAL_OK)
  {
    Error_Handler();
  }

  /* USER CODE BEGIN Check_RTC_BKUP */

  /* USER CODE END Check_RTC_BKUP */

  /** Initialize RTC and set the Time and Date
  */
  sTime.Hours = 0x0;
  sTime.Minutes = 0x0;
  sTime.Seconds = 0x0;
  sTime.DayLightSaving = RTC_DAYLIGHTSAVING_NONE;
  sTime.StoreOperation = RTC_STOREOPERATION_RESET;
  if (HAL_RTC_SetTime(&hrtc, &sTime, RTC_FORMAT_BCD) != HAL_OK)
  {
    Error_Handler();
  }
  sDate.WeekDay = RTC_WEEKDAY_MONDAY;
  sDate.Month = RTC_MONTH_JANUARY;
  sDate.Date = 0x1;
  sDate.Year = 0x0;

  if (HAL_RTC_SetDate(&hrtc, &sDate, RTC_FORMAT_BCD) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN RTC_Init 2 */

  /* USER CODE END RTC_Init 2 */

}

/**
  * @brief SPI1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_SPI1_Init(void)
{

  /* USER CODE BEGIN SPI1_Init 0 */

  /* USER CODE END SPI1_Init 0 */

  /* USER CODE BEGIN SPI1_Init 1 */

  /* USER CODE END SPI1_Init 1 */
  /* SPI1 parameter configuration*/
  hspi1.Instance = SPI1;
  hspi1.Init.Mode = SPI_MODE_MASTER;
  hspi1.Init.Direction = SPI_DIRECTION_2LINES;
  hspi1.Init.DataSize = SPI_DATASIZE_8BIT;
  hspi1.Init.CLKPolarity = SPI_POLARITY_LOW;
  hspi1.Init.CLKPhase = SPI_PHASE_1EDGE;
  hspi1.Init.NSS = SPI_NSS_SOFT;
  hspi1.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_256;
  hspi1.Init.FirstBit = SPI_FIRSTBIT_MSB;
  hspi1.Init.TIMode = SPI_TIMODE_DISABLE;
  hspi1.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  hspi1.Init.CRCPolynomial = 10;
  if (HAL_SPI_Init(&hspi1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN SPI1_Init 2 */

  /* USER CODE END SPI1_Init 2 */

}

/**
  * @brief TIM3 Initialization Function
  * @param None
  * @retval None
  */
static void MX_TIM3_Init(void)
{

  /* USER CODE BEGIN TIM3_Init 0 */

  /* USER CODE END TIM3_Init 0 */

  TIM_ClockConfigTypeDef sClockSourceConfig = {0};
  TIM_MasterConfigTypeDef sMasterConfig = {0};
  TIM_OC_InitTypeDef sConfigOC = {0};

  /* USER CODE BEGIN TIM3_Init 1 */

  /* USER CODE END TIM3_Init 1 */
  htim3.Instance = TIM3;
  htim3.Init.Prescaler = 0;
  htim3.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim3.Init.Period = 7000-1;
  htim3.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  htim3.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_DISABLE;
  if (HAL_TIM_Base_Init(&htim3) != HAL_OK)
  {
    Error_Handler();
  }
  sClockSourceConfig.ClockSource = TIM_CLOCKSOURCE_INTERNAL;
  if (HAL_TIM_ConfigClockSource(&htim3, &sClockSourceConfig) != HAL_OK)
  {
    Error_Handler();
  }
  if (HAL_TIM_OC_Init(&htim3) != HAL_OK)
  {
    Error_Handler();
  }
  sMasterConfig.MasterOutputTrigger = TIM_TRGO_UPDATE;
  sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
  if (HAL_TIMEx_MasterConfigSynchronization(&htim3, &sMasterConfig) != HAL_OK)
  {
    Error_Handler();
  }
  sConfigOC.OCMode = TIM_OCMODE_TOGGLE;
  sConfigOC.Pulse = 0;
  sConfigOC.OCPolarity = TIM_OCPOLARITY_HIGH;
  sConfigOC.OCFastMode = TIM_OCFAST_DISABLE;
  if (HAL_TIM_OC_ConfigChannel(&htim3, &sConfigOC, TIM_CHANNEL_1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN TIM3_Init 2 */

  /* USER CODE END TIM3_Init 2 */
  HAL_TIM_MspPostInit(&htim3);

}

/**
  * @brief USART2 Initialization Function
  * @param None
  * @retval None
  */
static void MX_USART2_UART_Init(void)
{

  /* USER CODE BEGIN USART2_Init 0 */

  /* USER CODE END USART2_Init 0 */

  /* USER CODE BEGIN USART2_Init 1 */

  /* USER CODE END USART2_Init 1 */
  huart2.Instance = USART2;
  huart2.Init.BaudRate = 115200;
  huart2.Init.WordLength = UART_WORDLENGTH_8B;
  huart2.Init.StopBits = UART_STOPBITS_1;
  huart2.Init.Parity = UART_PARITY_NONE;
  huart2.Init.Mode = UART_MODE_TX_RX;
  huart2.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart2.Init.OverSampling = UART_OVERSAMPLING_16;
  if (HAL_UART_Init(&huart2) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN USART2_Init 2 */

  /* USER CODE END USART2_Init 2 */

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOH_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOC, GPIO_PIN_5, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIO_GPIO_Port, GPIO_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOB9_GPIO_Port, GPIOB9_Pin, GPIO_PIN_SET);

  /*Configure GPIO pin : B1_Pin */
  GPIO_InitStruct.Pin = B1_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_FALLING;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(B1_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pin : PC5 */
  GPIO_InitStruct.Pin = GPIO_PIN_5;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

  /*Configure GPIO pins : GPIO_Pin GPIOB9_Pin */
  GPIO_InitStruct.Pin = GPIO_Pin|GPIOB9_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /* EXTI interrupt init*/
  HAL_NVIC_SetPriority(EXTI15_10_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(EXTI15_10_IRQn);

}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  __disable_irq();
  while (1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */
